#include "Utils.h"
#include "Win32Wrapper.h"
#include "Win32FileWrapper.h"

CWin32FileWrapper::CWin32FileWrapper(NUtils::FileType fileType, LPSTR lpszName)
: CWin32Wrapper(fileType)
{
	m_hFile = ::CreateFile(lpszName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL);

	if (INVALID_HANDLE_VALUE == m_hFile)
	{
		m_Type = NUtils::TYPE_UNKNOWN;
	}
}

CWin32FileWrapper::~CWin32FileWrapper()
{
	if (INVALID_HANDLE_VALUE != m_hFile)
	{
		CloseHandle(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
	}
}

FLAC__StreamDecoderReadStatus CWin32FileWrapper::v_easyflac_read_func(const EasyFLAC__StreamDecoder *decoder, FLAC__byte buffer[], unsigned *bytes)
{
	if (INVALID_HANDLE_VALUE != m_hFile)
	{
		DWORD dwBytesToRead = *bytes;
		DWORD dwBytesRead = 0;

		if (::ReadFile(m_hFile, buffer, dwBytesToRead, &dwBytesRead, NULL))
		{
			*bytes = dwBytesRead;
			return (dwBytesRead ? FLAC__STREAM_DECODER_READ_STATUS_CONTINUE
								: FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM);
		}
	}

	*bytes = 0;
	return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
}

size_t CWin32FileWrapper::v_vorbisfile_read_func(void *ptr, size_t size, size_t nmemb)
{
	if (INVALID_HANDLE_VALUE != m_hFile)
	{
		DWORD dwBytesToRead = size * nmemb;
		DWORD dwBytesRead = 0;

		if (::ReadFile(m_hFile, ptr, dwBytesToRead, &dwBytesRead, NULL))
		{
			errno = 0;
			return dwBytesRead; // If dwBYtesRead==0 then we are signally the end of the input.
		}
	}

	errno = 1;
	return 0; // Indicate a read error.
}

int CWin32FileWrapper::v_vorbisfile_seek_func(ogg_int64_t offset, int whence)
{
	if (INVALID_HANDLE_VALUE != m_hFile)
	{
		// Win9x will have errors seeking large files. Blah. An error is an error.
		LARGE_INTEGER liOffset;
		liOffset.QuadPart = offset;

		LONG lOffsetLow = liOffset.LowPart;
		LONG lOffsetHigh = liOffset.HighPart;

		DWORD dwMoveMethod;

		switch(whence)
		{
		default:
		case SEEK_SET:	dwMoveMethod = FILE_BEGIN;		break;
		case SEEK_CUR:	dwMoveMethod = FILE_CURRENT;	break;
		case SEEK_END:	dwMoveMethod = FILE_END;		break;
		}

		DWORD dwResult = ::SetFilePointer(m_hFile, lOffsetLow, &lOffsetHigh, dwMoveMethod);

		if (INVALID_SET_FILE_POINTER != dwResult || NO_ERROR == GetLastError())
		{
			return 0; // success
		}
	}

	return -1; // error
}

long CWin32FileWrapper::v_vorbisfile_tell_func()
{
	if (INVALID_HANDLE_VALUE != m_hFile)
	{
		// Dunno what the point of this returning a 32-bit long is when the seek call takes a 64-bit input.

		DWORD dwResult = ::SetFilePointer(m_hFile, 0, NULL, FILE_CURRENT);

		if (INVALID_SET_FILE_POINTER != dwResult)
		{
			return dwResult; // Success
		}
	}

	return -1; // Error.
}
