#include "Utils.h"
#include "Win32Wrapper.h"
#include "Win32StreamWrapper.h"

CWin32StreamWrapper::CWin32StreamWrapper(NUtils::FileType fileType, IStream *pStream, bool bSeekable)
: CWin32Wrapper(fileType), m_pStream(pStream), m_bSeekable(bSeekable)
{
}

CWin32StreamWrapper::~CWin32StreamWrapper()
{
}

FLAC__StreamDecoderReadStatus CWin32StreamWrapper::v_easyflac_read_func(const EasyFLAC__StreamDecoder *decoder, FLAC__byte buffer[], unsigned *bytes)
{
	ULONG ulBytesToRead = *bytes;
	ULONG ulBytesRead = 0;

	if (S_OK == m_pStream->Read(buffer, ulBytesToRead, &ulBytesRead))
	{
		*bytes = ulBytesRead;
		return (ulBytesRead ? FLAC__STREAM_DECODER_READ_STATUS_CONTINUE
							: FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM);
	}

	*bytes = 0;
	return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
}

size_t CWin32StreamWrapper::v_vorbisfile_read_func(void *ptr, size_t size, size_t nmemb)
{
	ULONG ulBytesToRead = size;
	ulBytesToRead *= nmemb;

	ULONG ulBytesRead = 0;

	if (S_OK == m_pStream->Read(ptr, ulBytesToRead, &ulBytesRead))
	{
		errno = 0;
		return ulBytesRead;
	}

	errno = 1;
	return 0;
}

int CWin32StreamWrapper::v_vorbisfile_seek_func(ogg_int64_t offset, int whence)
{
	if (m_bSeekable)
	{
		LARGE_INTEGER liOffset;
		liOffset.QuadPart = offset;

		DWORD dwOrigin;

		switch(whence)
		{
		default:
		case SEEK_SET:	dwOrigin = STREAM_SEEK_SET;	break;
		case SEEK_CUR:	dwOrigin = STREAM_SEEK_CUR;	break;
		case SEEK_END:	dwOrigin = STREAM_SEEK_END;	break;
		}

		if (S_OK == m_pStream->Seek(liOffset, dwOrigin, NULL))
		{
			return 0; // success
		}
	}

	return -1; // error
}

long CWin32StreamWrapper::v_vorbisfile_tell_func()
{
	if (m_bSeekable)
	{
		// Dunno what the point of this returning a long is when the seek call takes a 64-bit number.

		LARGE_INTEGER liSeek;
		liSeek.QuadPart = 0;

		ULARGE_INTEGER uliPos;
		uliPos.QuadPart = 0;

		if (S_OK == m_pStream->Seek(liSeek, STREAM_SEEK_CUR, &uliPos))
		{
			return uliPos.LowPart; // success
		}
	}

	return -1; // error
}
