#include "Utils.h"
#include "Win32Wrapper.h"
#include "Win32FileWrapper.h"
#include "Win32StreamWrapper.h"

extern "C"
{
	__declspec(dllexport) BOOL DVP_IdentifyA(LPVIEWERPLUGININFOA lpVPInfo);
	__declspec(dllexport) BOOL DVP_IdentifyFileBytesA(HWND hWnd,LPTSTR lpszName,LPBYTE lpData,UINT uiDataSize,LPVIEWERPLUGINFILEINFO lpVPFileInfo,DWORD dwStreamFlags);
	__declspec(dllexport) BOOL DVP_GetFileInfoFileA(HWND hWnd,LPTSTR lpszName,LPVIEWERPLUGINFILEINFO lpVPFileInfo,LPDVPFILEINFOHEADER lpFIH,HANDLE hAbortEvent);
	__declspec(dllexport) BOOL DVP_GetFileInfoFileStreamA(HWND hWnd,LPSTREAM lpStream,LPTSTR lpszName,LPVIEWERPLUGINFILEINFO lpVPFileInfo,LPDVPFILEINFOHEADER lpFIH,DWORD dwStreamFlags);
};

// {5B53809B-8B7D-4ad8-839F-679E14FADC2B}
static const GUID GUIDPlugin_OGG =
{ 0x5b53809b, 0x8b7d, 0x4ad8, { 0x83, 0x9f, 0x67, 0x9e, 0x14, 0xfa, 0xdc, 0x2b } };

HINSTANCE g_hModuleInstance=0;

extern "C" int WINAPI DllMain(HINSTANCE hInstance,DWORD dwReason,LPVOID)
{
	if (DLL_PROCESS_ATTACH == dwReason)
	{
		// Save DLL instance handle
		g_hModuleInstance=hInstance;
	}

	return 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Identify the viewer plugin to DOpus
BOOL DVP_IdentifyA(LPVIEWERPLUGININFOA lpVPInfo)
{
	BOOL bResult = FALSE;

	// DVPFIF_ExtensionsOnly;
	lpVPInfo->dwFlags=DVPFIF_CanHandleStreams|DVPFIF_CanHandleBytes|DVPFIF_ProvideFileInfo|DVPFIF_NoThumbnails|DVPFIF_NoSlowFiles;
	lpVPInfo->lpszHandleExts = TEXT(".ogg;.flac");
	lpVPInfo->uiMajorFileType=DVPMajorType_Sound;
	lpVPInfo->idPlugin=GUIDPlugin_OGG;
	lpVPInfo->dwlMinFileSize = 32;
	lpVPInfo->dwlMaxFileSize = 0;

	// Get the remaining information from the version resource.

	TCHAR szModName[_MAX_PATH + 1];
	DWORD dwVerSize;
	DWORD dwUnused;

	if (NULL != g_hModuleInstance
	&&	GetModuleFileName(g_hModuleInstance, szModName, _MAX_PATH)
	&&	(0 != (dwVerSize = GetFileVersionInfoSize(szModName, &dwUnused))))
	{
		BYTE *pVerData = new BYTE[dwVerSize];

		if (GetFileVersionInfo(szModName, NULL, dwVerSize, pVerData))
		{
			LPVOID pBuffer;
			UINT bufLen;

			// Find out language of version resource
			if (VerQueryValue(pVerData, _T("\\VarFileInfo\\Translation"), &pBuffer, &bufLen)
			&&	4 == bufLen)
			{
				BYTE *pLang = reinterpret_cast<BYTE*>(pBuffer);

				if (NUtils::CopyVersionNumber(&lpVPInfo->dwVersionHigh,	&lpVPInfo->dwVersionLow,	_T("FileVersion"),		pLang, pVerData)
				&&	NUtils::CopyVersionString(lpVPInfo->lpszName,		lpVPInfo->cchNameMax,		_T("InternalName"),		pLang, pVerData)
				&&	NUtils::CopyVersionString(lpVPInfo->lpszDescription,lpVPInfo->cchDescriptionMax,_T("FileDescription"),	pLang, pVerData)
				&&	NUtils::CopyVersionString(lpVPInfo->lpszURL,		lpVPInfo->cchURLMax,		_T("CompanyName"   ),	pLang, pVerData)
				&&  NUtils::CopyVersionString(lpVPInfo->lpszCopyright,	lpVPInfo->cchCopyrightMax,	_T("LegalCopyright"),	pLang, pVerData))
				{
					bResult = TRUE;
				}
			}
		}

		delete [] pVerData;
	}

	return bResult;
}

// Identify a file or stream based on the first ~256 bytes.
BOOL DVP_IdentifyFileBytesA(HWND hWnd,LPTSTR lpszName,LPBYTE lpData,UINT uiDataSize,LPVIEWERPLUGINFILEINFO lpVPFileInfo,DWORD dwStreamFlags)
{
	BOOL bResult = FALSE;

	NUtils::FileType fileType = NUtils::GetTypeFromBuffer(lpData, uiDataSize);

	if (NUtils::TYPE_UNKNOWN != fileType)
	{
		lpVPFileInfo->dwFlags=DVPFIF_CanReturnFileInfo;
		lpVPFileInfo->wMajorType=DVPMajorType_Sound;
		lpVPFileInfo->dwPrivateData[0] = static_cast<DWORD>(fileType); // Stash the type away for later.

		bResult = TRUE;
	}

	return bResult;
}

// Get info from a file
BOOL DVP_GetFileInfoFileA(HWND hWnd,LPTSTR lpszName,LPVIEWERPLUGINFILEINFO lpVPFileInfo,LPDVPFILEINFOHEADER lpFIH,HANDLE hAbortEvent)
{
	if (lpFIH->uiMajorType!=DVPMajorType_Sound)
	{
		return FALSE;
	}
	else
	{
		CWin32FileWrapper wrapper( static_cast<NUtils::FileType>(lpVPFileInfo->dwPrivateData[0]), lpszName );

		return wrapper.Process( reinterpret_cast<LPDVPFILEINFOMUSIC>(lpFIH) );
	}
}

// Get file info from a stream-based file
BOOL DVP_GetFileInfoFileStreamA(HWND hWnd,LPSTREAM lpStream,LPTSTR lpszName,LPVIEWERPLUGINFILEINFO lpVPFileInfo,LPDVPFILEINFOHEADER lpFIH,DWORD dwStreamFlags)
{
	if (lpFIH->uiMajorType!=DVPMajorType_Sound)
	{
		return FALSE;
	}
	else
	{
		CWin32StreamWrapper wrapper( static_cast<NUtils::FileType>(lpVPFileInfo->dwPrivateData[0]),
										lpStream, !(dwStreamFlags & DVPSF_NoRandomSeek) );

		return wrapper.Process( reinterpret_cast<LPDVPFILEINFOMUSIC>(lpFIH) );
	}
}
